@@ -1,5 +1,9 @@
 #![doc = include_str!("../README.md")]
 
+#[cfg(any(feature = "tree-sitter-highlight", feature = "tree-sitter-tags"))]
+use std::ops::Range;
+#[cfg(feature = "tree-sitter-highlight")]
+use std::sync::Mutex;
 use std::{
     collections::HashMap,
     env,
@@ -7,22 +11,28 @@ use std::{
     fs,
     io::{BufRead, BufReader},
     mem,
-    ops::Range,
     path::{Path, PathBuf},
     process::Command,
-    sync::Mutex,
     time::SystemTime,
 };
 
-use anyhow::{anyhow, Context, Error, Result};
+#[cfg(any(feature = "tree-sitter-highlight", feature = "tree-sitter-tags"))]
+use anyhow::Error;
+use anyhow::{anyhow, Context, Result};
 use fs4::FileExt;
 use indoc::indoc;
 use libloading::{Library, Symbol};
 use once_cell::unsync::OnceCell;
 use regex::{Regex, RegexBuilder};
 use serde::{Deserialize, Deserializer, Serialize};
-use tree_sitter::{Language, QueryError, QueryErrorKind};
+use tree_sitter::Language;
+#[cfg(any(feature = "tree-sitter-highlight", feature = "tree-sitter-tags"))]
+use tree_sitter::QueryError;
+#[cfg(feature = "tree-sitter-highlight")]
+use tree_sitter::QueryErrorKind;
+#[cfg(feature = "tree-sitter-highlight")]
 use tree_sitter_highlight::HighlightConfiguration;
+#[cfg(feature = "tree-sitter-tags")]
 use tree_sitter_tags::{Error as TagsError, TagsConfiguration};
 
 pub const EMSCRIPTEN_TAG: &str = concat!("docker.io/emscripten/emsdk:", env!("EMSCRIPTEN_VERSION"));
@@ -83,6 +93,7 @@ impl Config {
 }
 
 const BUILD_TARGET: &str = env!("BUILD_TARGET");
+const BUILD_HOST: &str = env!("BUILD_HOST");
 
 pub struct LanguageConfiguration<'a> {
     pub scope: Option<String>,
@@ -97,9 +108,13 @@ pub struct LanguageConfiguration<'a> {
     pub tags_filenames: Option<Vec<String>>,
     pub language_name: String,
     language_id: usize,
+    #[cfg(feature = "tree-sitter-highlight")]
     highlight_config: OnceCell<Option<HighlightConfiguration>>,
+    #[cfg(feature = "tree-sitter-tags")]
     tags_config: OnceCell<Option<TagsConfiguration>>,
+    #[cfg(feature = "tree-sitter-highlight")]
     highlight_names: &'a Mutex<Vec<String>>,
+    #[cfg(feature = "tree-sitter-highlight")]
     use_all_highlight_names: bool,
 }
 
@@ -110,9 +125,12 @@ pub struct Loader {
     language_configuration_ids_by_file_type: HashMap<String, Vec<usize>>,
     language_configuration_in_current_path: Option<usize>,
     language_configuration_ids_by_first_line_regex: HashMap<String, Vec<usize>>,
+    #[cfg(feature = "tree-sitter-highlight")]
     highlight_names: Box<Mutex<Vec<String>>>,
+    #[cfg(feature = "tree-sitter-highlight")]
     use_all_highlight_names: bool,
     debug_build: bool,
+    sanitize_build: bool,
 
     #[cfg(feature = "wasm")]
     wasm_store: Mutex<Option<tree_sitter::WasmStore>>,
@@ -126,6 +144,7 @@ pub struct CompileConfig<'a> {
     pub external_files: Option<&'a [PathBuf]>,
     pub output_path: Option<PathBuf>,
     pub flags: &'a [&'a str],
+    pub sanitize: bool,
     pub name: String,
 }
 
@@ -143,7 +162,8 @@ impl<'a> CompileConfig<'a> {
             scanner_path: None,
             external_files: externals,
             output_path,
-            flags: &["TREE_SITTER_REUSE_ALLOCATOR", "TREE_SITTER_INTERNAL_BUILD"],
+            flags: &[],
+            sanitize: false,
             name: String::new(),
         }
     }
@@ -173,15 +193,19 @@ impl Loader {
             language_configuration_ids_by_file_type: HashMap::new(),
             language_configuration_in_current_path: None,
             language_configuration_ids_by_first_line_regex: HashMap::new(),
+            #[cfg(feature = "tree-sitter-highlight")]
             highlight_names: Box::new(Mutex::new(Vec::new())),
+            #[cfg(feature = "tree-sitter-highlight")]
             use_all_highlight_names: true,
             debug_build: false,
+            sanitize_build: false,
 
             #[cfg(feature = "wasm")]
             wasm_store: Mutex::default(),
         }
     }
 
+    #[cfg(feature = "tree-sitter-highlight")]
     pub fn configure_highlights(&mut self, names: &[String]) {
         self.use_all_highlight_names = false;
         let mut highlights = self.highlight_names.lock().unwrap();
@@ -190,6 +214,7 @@ impl Loader {
     }
 
     #[must_use]
+    #[cfg(feature = "tree-sitter-highlight")]
     pub fn highlight_names(&self) -> Vec<String> {
         self.highlight_names.lock().unwrap().clone()
     }
@@ -410,10 +435,19 @@ impl Loader {
         struct GrammarJSON {
             name: String,
         }
-        let mut grammar_file =
-            fs::File::open(grammar_path).with_context(|| "Failed to read grammar.json")?;
+        let mut grammar_file = fs::File::open(&grammar_path).with_context(|| {
+            format!(
+                "Failed to read grammar.json file at the following path:\n{:?}",
+                &grammar_path
+            )
+        })?;
         let grammar_json: GrammarJSON = serde_json::from_reader(BufReader::new(&mut grammar_file))
-            .with_context(|| "Failed to parse grammar.json")?;
+            .with_context(|| {
+                format!(
+                    "Failed to parse grammar.json file at the following path:\n{:?}",
+                    &grammar_path
+                )
+            })?;
 
         config.name = grammar_json.name;
 
@@ -430,6 +464,11 @@ impl Loader {
             lib_name.push_str(".debug._");
         }
 
+        if self.sanitize_build {
+            lib_name.push_str(".sanitize._");
+            config.sanitize = true;
+        }
+
         if config.output_path.is_none() {
             fs::create_dir_all(&self.parser_lib_path)?;
         }
@@ -474,6 +513,7 @@ impl Loader {
             if recompile {
                 self.compile_parser_to_wasm(
                     &config.name,
+                    None,
                     config.src_path,
                     config
                         .scanner_path
@@ -561,16 +601,39 @@ impl Loader {
         lock_file: &fs::File,
         lock_path: &Path,
     ) -> Result<(), Error> {
-        let mut cc = cc::Build::new();
-        cc.cpp(true)
-            .opt_level(2)
+        let mut cc_config = cc::Build::new();
+        cc_config
             .cargo_metadata(false)
             .cargo_warnings(false)
             .target(BUILD_TARGET)
-            .host(BUILD_TARGET)
-            .flag_if_supported("-Werror=implicit-function-declaration");
-        let compiler = cc.get_compiler();
+            .host(BUILD_HOST)
+            .debug(self.debug_build)
+            .file(&config.parser_path)
+            .includes(&config.header_paths);
+
+        if let Some(scanner_path) = config.scanner_path.as_ref() {
+            if scanner_path.extension() != Some("c".as_ref()) {
+                cc_config.cpp(true);
+                eprintln!("Warning: Using a C++ scanner is now deprecated. Please migrate your scanner code to C, as C++ support will be removed in the near future.");
+            } else {
+                cc_config.std("c11");
+            }
+            cc_config.file(scanner_path);
+        }
+
+        if self.debug_build {
+            cc_config.opt_level(0).extra_warnings(true);
+        } else {
+            cc_config.opt_level(2).extra_warnings(false);
+        }
+
+        for flag in config.flags {
+            cc_config.define(flag, None);
+        }
+
+        let compiler = cc_config.get_compiler();
         let mut command = Command::new(compiler.path());
+        command.args(compiler.args());
         for (key, value) in compiler.env() {
             command.env(key, value);
         }
@@ -578,72 +641,24 @@ impl Loader {
         let output_path = config.output_path.as_ref().unwrap();
 
         if compiler.is_like_msvc() {
-            command.args(["/nologo", "/LD"]);
-
-            for path in &config.header_paths {
-                command.arg(format!("/I{}", path.to_string_lossy()));
-            }
-
-            if self.debug_build {
-                command.arg("/Od");
-            } else {
-                command.arg("/O2");
-            }
-            command.arg(&config.parser_path);
-
-            if let Some(scanner_path) = config.scanner_path.as_ref() {
-                if scanner_path.extension() != Some("c".as_ref()) {
-                    eprintln!("Warning: Using a C++ scanner is now deprecated. Please migrate your scanner code to C, as C++ support will be removed in the near future.");
-                }
-
-                command.arg(scanner_path);
-            }
-            command
-                .arg("/utf-8")
-                .arg("/link")
-                .arg(format!("/out:{}", output_path.to_str().unwrap()));
+            let out = format!("-out:{}", output_path.to_str().unwrap());
+            command.arg(if self.debug_build { "-LDd" } else { "-LD" });
+            command.arg("-utf-8");
+            command.args(cc_config.get_files());
+            command.arg("-link").arg(out);
         } else {
-            command
-                .arg("-shared")
-                .arg("-fno-exceptions")
-                .arg("-g")
-                .arg("-o")
-                .arg(output_path);
-
-            for path in &config.header_paths {
-                command.arg(format!("-I{}", path.to_string_lossy()));
-            }
-
-            if !cfg!(windows) {
-                command.arg("-fPIC");
-            }
-
-            if self.debug_build {
-                command.arg("-O0");
+            command.arg("-Werror=implicit-function-declaration");
+            if cfg!(any(target_os = "macos", target_os = "ios")) {
+                command.arg("-dynamiclib");
+                // TODO: remove when supported
+                command.arg("-UTREE_SITTER_REUSE_ALLOCATOR");
             } else {
-                command.arg("-O2");
+                command.arg("-shared");
             }
-
-            if let Some(scanner_path) = config.scanner_path.as_ref() {
-                if scanner_path.extension() == Some("c".as_ref()) {
-                    command.arg("-xc").arg("-std=c11").arg(scanner_path);
-                } else {
-                    eprintln!("Warning: Using a C++ scanner is now deprecated. Please migrate your scanner code to C, as C++ support will be removed in the near future.");
-                    command.arg(scanner_path);
-                }
-            }
-            command.arg("-xc").arg(&config.parser_path);
+            command.args(cc_config.get_files());
+            command.arg("-o").arg(output_path);
         }
 
-        // For conditional compilation of external scanner code when
-        // used internally by `tree-sitter parse` and other sub commands.
-        command.arg("-DTREE_SITTER_INTERNAL_BUILD");
-
-        // Always use the same allocator in the CLI as any scanner, useful for debugging and
-        // tracking memory leaks in tests.
-        #[cfg(not(any(target_os = "macos", target_os = "ios")))]
-        command.arg("-DTREE_SITTER_REUSE_ALLOCATOR");
-
         let output = command.output().with_context(|| {
             format!("Failed to execute the C compiler with the following command:\n{command:?}")
         })?;
@@ -651,20 +666,24 @@ impl Loader {
         lock_file.unlock()?;
         fs::remove_file(lock_path)?;
 
-        if !output.status.success() {
-            return Err(anyhow!(
+        if output.status.success() {
+            Ok(())
+        } else {
+            Err(anyhow!(
                 "Parser compilation failed.\nStdout: {}\nStderr: {}",
                 String::from_utf8_lossy(&output.stdout),
                 String::from_utf8_lossy(&output.stderr)
-            ));
+            ))
         }
-
-        Ok(())
     }
 
     #[cfg(unix)]
     fn check_external_scanner(&self, name: &str, library_path: &Path) -> Result<()> {
-        let prefix = if cfg!(target_os = "macos") { "_" } else { "" };
+        let prefix = if cfg!(any(target_os = "macos", target_os = "ios")) {
+            "_"
+        } else {
+            ""
+        };
         let mut must_have = vec![
             format!("{prefix}tree_sitter_{name}_external_scanner_create"),
             format!("{prefix}tree_sitter_{name}_external_scanner_destroy"),
@@ -744,6 +763,7 @@ impl Loader {
     pub fn compile_parser_to_wasm(
         &self,
         language_name: &str,
+        root_path: Option<&Path>,
         src_path: &Path,
         scanner_filename: Option<&Path>,
         output_path: &Path,
@@ -756,6 +776,7 @@ impl Loader {
             Podman,
         }
 
+        let root_path = root_path.unwrap_or(src_path);
         let emcc_name = if cfg!(windows) { "emcc.bat" } else { "emcc" };
 
         // Order of preference: emscripten > docker > podman > error
@@ -794,10 +815,18 @@ impl Loader {
                 };
                 command.args(["run", "--rm"]);
 
-                // Mount the parser directory as a volume
-                command.args(["--workdir", "/src"]);
+                // The working directory is the directory containing the parser itself
+                let workdir = if root_path == src_path {
+                    PathBuf::from("/src")
+                } else {
+                    let mut path = PathBuf::from("/src");
+                    path.push(src_path.strip_prefix(root_path).unwrap());
+                    path
+                };
+                command.args(["--workdir", &workdir.to_string_lossy()]);
 
-                let mut volume_string = OsString::from(&src_path);
+                // Mount the root directory as a volume, which is the repo root
+                let mut volume_string = OsString::from(&root_path);
                 volume_string.push(":/src:Z");
                 command.args([OsStr::new("--volume"), &volume_string]);
 
@@ -880,6 +909,7 @@ impl Loader {
     }
 
     #[must_use]
+    #[cfg(feature = "tree-sitter-highlight")]
     pub fn highlight_config_for_injection_string<'a>(
         &'a self,
         string: &str,
@@ -1032,9 +1062,13 @@ impl Loader {
                         locals_filenames: config_json.locals.into_vec(),
                         tags_filenames: config_json.tags.into_vec(),
                         highlights_filenames: config_json.highlights.into_vec(),
+                        #[cfg(feature = "tree-sitter-highlight")]
                         highlight_config: OnceCell::new(),
+                        #[cfg(feature = "tree-sitter-tags")]
                         tags_config: OnceCell::new(),
+                        #[cfg(feature = "tree-sitter-highlight")]
                         highlight_names: &self.highlight_names,
+                        #[cfg(feature = "tree-sitter-highlight")]
                         use_all_highlight_names: self.use_all_highlight_names,
                     };
 
@@ -1089,9 +1123,13 @@ impl Loader {
                 locals_filenames: None,
                 highlights_filenames: None,
                 tags_filenames: None,
+                #[cfg(feature = "tree-sitter-highlight")]
                 highlight_config: OnceCell::new(),
+                #[cfg(feature = "tree-sitter-tags")]
                 tags_config: OnceCell::new(),
+                #[cfg(feature = "tree-sitter-highlight")]
                 highlight_names: &self.highlight_names,
+                #[cfg(feature = "tree-sitter-highlight")]
                 use_all_highlight_names: self.use_all_highlight_names,
             };
             self.language_configurations.push(unsafe {
@@ -1151,12 +1189,16 @@ impl Loader {
         }
     }
 
-    pub fn use_debug_build(&mut self, flag: bool) {
+    pub fn debug_build(&mut self, flag: bool) {
         self.debug_build = flag;
     }
 
+    pub fn sanitize_build(&mut self, flag: bool) {
+        self.sanitize_build = flag;
+    }
+
     #[cfg(feature = "wasm")]
-    pub fn use_wasm(&mut self, engine: tree_sitter::wasmtime::Engine) {
+    pub fn use_wasm(&mut self, engine: &tree_sitter::wasmtime::Engine) {
         *self.wasm_store.lock().unwrap() = Some(tree_sitter::WasmStore::new(engine).unwrap());
     }
 
@@ -1174,6 +1216,7 @@ impl Loader {
 }
 
 impl<'a> LanguageConfiguration<'a> {
+    #[cfg(feature = "tree-sitter-highlight")]
     pub fn highlight_config(
         &self,
         language: Language,
@@ -1184,14 +1227,14 @@ impl<'a> LanguageConfiguration<'a> {
                 Some(
                     paths
                         .iter()
-                        .filter(|p| p.ends_with("highlights.scm"))
+                        .filter(|p| p.ends_with("tree-sitter-highlights.scm"))
                         .cloned()
                         .collect::<Vec<_>>(),
                 ),
                 Some(
                     paths
                         .iter()
-                        .filter(|p| p.ends_with("tags.scm"))
+                        .filter(|p| p.ends_with("tree-sitter-tags.scm"))
                         .cloned()
                         .collect::<Vec<_>>(),
                 ),
@@ -1213,7 +1256,7 @@ impl<'a> LanguageConfiguration<'a> {
                     } else {
                         self.highlights_filenames.as_deref()
                     },
-                    "highlights.scm",
+                    "tree-sitter-highlights.scm",
                 )?;
                 let (injections_query, injection_ranges) = self.read_queries(
                     if injections_filenames.is_some() {
@@ -1285,11 +1328,12 @@ impl<'a> LanguageConfiguration<'a> {
             .map(Option::as_ref)
     }
 
+    #[cfg(feature = "tree-sitter-tags")]
     pub fn tags_config(&self, language: Language) -> Result<Option<&TagsConfiguration>> {
         self.tags_config
             .get_or_try_init(|| {
                 let (tags_query, tags_ranges) =
-                    self.read_queries(self.tags_filenames.as_deref(), "tags.scm")?;
+                    self.read_queries(self.tags_filenames.as_deref(), "tree-sitter-tags.scm")?;
                 let (locals_query, locals_ranges) =
                     self.read_queries(self.locals_filenames.as_deref(), "locals.scm")?;
                 if tags_query.is_empty() {
@@ -1323,6 +1367,7 @@ impl<'a> LanguageConfiguration<'a> {
             .map(Option::as_ref)
     }
 
+    #[cfg(any(feature = "tree-sitter-highlight", feature = "tree-sitter-tags"))]
     fn include_path_in_query_error(
         mut error: QueryError,
         ranges: &[(String, Range<usize>)],
@@ -1336,12 +1381,13 @@ impl<'a> LanguageConfiguration<'a> {
             .unwrap_or_else(|| ranges.last().unwrap());
         error.offset = offset_within_section - range.start;
         error.row = source[range.start..offset_within_section]
-            .matches(|c| c == '\n')
+            .matches('\n')
             .count();
         Error::from(error).context(format!("Error in query file {path:?}"))
     }
 
     #[allow(clippy::type_complexity)]
+    #[cfg(any(feature = "tree-sitter-highlight", feature = "tree-sitter-tags"))]
     fn read_queries(
         &self,
         paths: Option<&[String]>,
@@ -1359,7 +1405,9 @@ impl<'a> LanguageConfiguration<'a> {
             }
         } else {
             // highlights.scm is needed to test highlights, and tags.scm to test tags
-            if default_path == "highlights.scm" || default_path == "tags.scm" {
+            if default_path == "tree-sitter-highlights.scm"
+                || default_path == "tree-sitter-tags.scm"
+            {
                 eprintln!(
                     indoc! {"
                         Warning: you should add a `{}` entry pointing to the highlights path in `tree-sitter` language list in the grammar's package.json
