@@ -223,6 +223,8 @@ function RuleBuilder(ruleMap) {
 }
 
 function grammar(baseGrammar, options) {
+  let inherits = undefined;
+
   if (!options) {
     options = baseGrammar;
     baseGrammar = {
@@ -237,6 +239,7 @@ function grammar(baseGrammar, options) {
     };
   } else {
     baseGrammar = baseGrammar.grammar;
+    inherits = baseGrammar.name;
   }
 
   let externals = baseGrammar.externals;
@@ -279,6 +282,14 @@ function grammar(baseGrammar, options) {
     throw new Error("Grammar's 'name' property must not start with a digit and cannot contain non-word characters.");
   }
 
+  if (inherits && typeof inherits !== "string") {
+    throw new Error("Base grammar's 'name' property must be a string.");
+  }
+
+  if (inherits && !/^[a-zA-Z_]\w*$/.test(name)) {
+    throw new Error("Base grammar's 'name' property must not start with a digit and cannot contain non-word characters.");
+  }
+
   const rules = Object.assign({}, baseGrammar.rules);
   if (options.rules) {
     if (typeof options.rules !== "object") {
@@ -290,7 +301,11 @@ function grammar(baseGrammar, options) {
       if (typeof ruleFn !== "function") {
         throw new Error(`Grammar rules must all be functions. '${ruleName}' rule is not.`);
       }
-      rules[ruleName] = normalize(ruleFn.call(ruleBuilder, ruleBuilder, baseGrammar.rules[ruleName]));
+      const rule = ruleFn.call(ruleBuilder, ruleBuilder, baseGrammar.rules[ruleName]);
+      if (rule === undefined) {
+        throw new Error(`Rule '${ruleName}' returned undefined.`);
+      }
+      rules[ruleName] = normalize(rule);
     }
   }
 
@@ -383,7 +398,12 @@ function grammar(baseGrammar, options) {
       throw new Error("Grammar's supertypes must be an array of rules.");
     }
 
-    supertypes = supertypeRules.map(symbol => symbol.name);
+    supertypes = supertypeRules.map(symbol => {
+      if (symbol.name === 'ReferenceError') {
+        throw new Error(`Supertype rule \`${symbol.symbol.name}\` is not defined.`);
+      }
+      return symbol.name;
+    });
   }
 
   let precedences = baseGrammar.precedences;
@@ -407,7 +427,20 @@ function grammar(baseGrammar, options) {
     throw new Error("Grammar must have at least one rule.");
   }
 
-  return { grammar: { name, word, rules, extras, conflicts, precedences, externals, inline, supertypes } };
+  return {
+    grammar: {
+      name,
+      inherits,
+      word,
+      rules,
+      extras,
+      conflicts,
+      precedences,
+      externals,
+      inline,
+      supertypes,
+    },
+  };
 }
 
 function checkArguments(args, ruleCount, caller, callerName, suffix = '', argType = 'rule') {
@@ -432,18 +465,32 @@ function checkPrecedence(value) {
   }
 }
 
-global.alias = alias;
-global.blank = blank;
-global.choice = choice;
-global.optional = optional;
-global.prec = prec;
-global.repeat = repeat;
-global.repeat1 = repeat1;
-global.seq = seq;
-global.sym = sym;
-global.token = token;
-global.grammar = grammar;
-global.field = field;
+function getEnv(name) {
+  if (globalThis.process) return process.env[name]; // Node/Bun
+  if (globalThis.Deno) return Deno.env.get(name); // Deno
+  throw Error("Unsupported JS runtime");
+}
 
-const result = require(process.env.TREE_SITTER_GRAMMAR_PATH);
-process.stdout.write(JSON.stringify(result.grammar, null, null));
+globalThis.alias = alias;
+globalThis.blank = blank;
+globalThis.choice = choice;
+globalThis.optional = optional;
+globalThis.prec = prec;
+globalThis.repeat = repeat;
+globalThis.repeat1 = repeat1;
+globalThis.seq = seq;
+globalThis.sym = sym;
+globalThis.token = token;
+globalThis.grammar = grammar;
+globalThis.field = field;
+
+const result = await import(getEnv("TREE_SITTER_GRAMMAR_PATH"));
+const output = JSON.stringify(result.default?.grammar ?? result.grammar);
+
+if (globalThis.process) { // Node/Bun
+  process.stdout.write(output);
+} else if (globalThis.Deno) { // Deno
+  Deno.stdout.writeSync(new TextEncoder().encode(output));
+} else {
+  throw Error("Unsupported JS runtime");
+}
