@@ -11,10 +11,15 @@ use glob::glob;
 use regex::Regex;
 use tree_sitter::{ffi, Parser, Point};
 use tree_sitter_cli::{
-    generate, highlight, logger,
+    fuzz::{
+        fuzz_language_corpus, FuzzOptions, EDIT_COUNT, ITERATION_COUNT, LOG_ENABLED,
+        LOG_GRAPH_ENABLED, START_SEED,
+    },
+    generate::{self, lookup_package_json_for_path},
+    highlight, logger,
     parse::{self, ParseFileOptions, ParseOutput},
-    playground, query, tags, test,
-    test::TestOptions,
+    playground, query, tags,
+    test::{self, TestOptions},
     test_highlight, test_tags, util, wasm,
 };
 use tree_sitter_config::Config;
@@ -35,6 +40,7 @@ enum Commands {
     BuildWasm(BuildWasm),
     Parse(Parse),
     Test(Test),
+    Fuzz(Fuzz),
     Query(Query),
     Highlight(Highlight),
     Tags(Tags),
@@ -91,7 +97,8 @@ struct Generate {
         long,
         value_name = "EXECUTABLE",
         env = "TREE_SITTER_JS_RUNTIME",
-        help = "The path to the JavaScript runtime to use for generating parsers"
+        default_value = "node",
+        help = "The name or path of the JavaScript runtime to use for generating parsers"
     )]
     pub js_runtime: Option<String>,
 }
@@ -113,11 +120,8 @@ struct Build {
     pub path: Option<String>,
     #[arg(long, help = "Make the parser reuse the same allocator as the library")]
     pub reuse_allocator: bool,
-    #[arg(
-        long,
-        help = "Build the parser with `TREE_SITTER_INTERNAL_BUILD` defined"
-    )]
-    pub internal_build: bool,
+    #[arg(long, short = '0', help = "Compile a parser in debug mode")]
+    pub debug: bool,
 }
 
 #[derive(Args)]
@@ -194,6 +198,9 @@ struct Parse {
     pub open_log: bool,
     #[arg(long, help = "The path to an alternative config.json file")]
     pub config_path: Option<PathBuf>,
+    #[arg(long, short = 'n', help = "Parse the contents of a specific test")]
+    #[clap(conflicts_with = "paths", conflicts_with = "paths_file")]
+    pub test_number: Option<u32>,
 }
 
 #[derive(Args)]
@@ -245,6 +252,27 @@ struct Test {
     pub open_log: bool,
     #[arg(long, help = "The path to an alternative config.json file")]
     pub config_path: Option<PathBuf>,
+    #[arg(long, help = "Force showing fields in test diffs")]
+    pub show_fields: bool,
+}
+
+#[derive(Args)]
+#[command(about = "Fuzz a parser", alias = "f")]
+struct Fuzz {
+    #[arg(long, short, help = "List of test names to skip")]
+    pub skip: Option<Vec<String>>,
+    #[arg(long, help = "Subdirectory to the language")]
+    pub subdir: Option<String>,
+    #[arg(long, short, help = "Maximum number of edits to perform per fuzz test")]
+    pub edits: Option<usize>,
+    #[arg(long, short, help = "Number of fuzzing iterations to run per test")]
+    pub iterations: Option<usize>,
+    #[arg(long, short, help = "Regex pattern to filter tests")]
+    pub filter: Option<Regex>,
+    #[arg(long, short, help = "Enable logging of graphs and input")]
+    pub log_graphs: bool,
+    #[arg(long, short, help = "Enable parser logging")]
+    pub log: bool,
 }
 
 #[derive(Args)]
@@ -409,6 +437,8 @@ fn run() -> Result<()> {
     let current_dir = env::current_dir().unwrap();
     let mut loader = loader::Loader::new()?;
 
+    let color = env::var("NO_COLOR").map_or(true, |v| v != "1");
+
     match command {
         Commands::InitConfig(_) => {
             if let Ok(Some(config_path)) = Config::find_config_file() {
@@ -453,7 +483,7 @@ fn run() -> Result<()> {
                 if let Some(path) = generate_options.libdir {
                     loader = loader::Loader::with_parser_lib_path(PathBuf::from(path));
                 }
-                loader.use_debug_build(generate_options.debug_build);
+                loader.debug_build(generate_options.debug_build);
                 loader.languages_at_path(&current_dir)?;
             }
         }
@@ -463,8 +493,11 @@ fn run() -> Result<()> {
                 let grammar_path =
                     current_dir.join(build_options.path.as_deref().unwrap_or_default());
                 let output_path = build_options.output.map(|path| current_dir.join(path));
+                let root_path = lookup_package_json_for_path(&grammar_path.join("package.json"))
+                    .map(|(p, _)| p.parent().unwrap().to_path_buf())?;
                 wasm::compile_language_to_wasm(
                     &loader,
+                    Some(&root_path),
                     &grammar_path,
                     &current_dir,
                     output_path,
@@ -493,15 +526,14 @@ fn run() -> Result<()> {
                         .with_extension(env::consts::DLL_EXTENSION)
                 };
 
-                let flags: &[&str] =
-                    match (build_options.reuse_allocator, build_options.internal_build) {
-                        (true, true) => {
-                            &["TREE_SITTER_REUSE_ALLOCATOR", "TREE_SITTER_INTERNAL_BUILD"]
-                        }
-                        (true, false) => &["TREE_SITTER_REUSE_ALLOCATOR"],
-                        (false, true) => &["TREE_SITTER_INTERNAL_BUILD"],
-                        (false, false) => &[""],
-                    };
+                let flags: &[&str] = match (build_options.reuse_allocator, build_options.debug) {
+                    (true, true) => &["TREE_SITTER_REUSE_ALLOCATOR", "TREE_SITTER_DEBUG"],
+                    (true, false) => &["TREE_SITTER_REUSE_ALLOCATOR"],
+                    (false, true) => &["TREE_SITTER_DEBUG"],
+                    (false, false) => &[],
+                };
+
+                loader.debug_build(build_options.debug);
 
                 let config = Config::load(None)?;
                 let loader_config = config.get()?;
@@ -515,8 +547,11 @@ fn run() -> Result<()> {
         Commands::BuildWasm(wasm_options) => {
             eprintln!("`build-wasm` is deprecated and will be removed in v0.24.0. You should use `build --wasm` instead");
             let grammar_path = current_dir.join(wasm_options.path.unwrap_or_default());
+            let root_path = lookup_package_json_for_path(&grammar_path.join("package.json"))
+                .map(|(p, _)| p.parent().unwrap().to_path_buf())?;
             wasm::compile_language_to_wasm(
                 &loader,
+                Some(&root_path),
                 &grammar_path,
                 &current_dir,
                 None,
@@ -551,25 +586,35 @@ fn run() -> Result<()> {
             let cancellation_flag = util::cancel_on_signal();
             let mut parser = Parser::new();
 
-            if parse_options.debug {
-                // For augmenting debug logging in external scanners
-                env::set_var("TREE_SITTER_DEBUG", "1");
-            }
-
-            loader.use_debug_build(parse_options.debug_build);
+            loader.debug_build(parse_options.debug_build);
 
             #[cfg(feature = "wasm")]
             if parse_options.wasm {
                 let engine = tree_sitter::wasmtime::Engine::default();
                 parser
-                    .set_wasm_store(tree_sitter::WasmStore::new(engine.clone()).unwrap())
+                    .set_wasm_store(tree_sitter::WasmStore::new(&engine).unwrap())
                     .unwrap();
-                loader.use_wasm(engine);
+                loader.use_wasm(&engine);
             }
 
             let timeout = parse_options.timeout.unwrap_or_default();
 
-            let paths = collect_paths(parse_options.paths_file.as_deref(), parse_options.paths)?;
+            let (paths, language) = if let Some(target_test) = parse_options.test_number {
+                let (test_path, language_names) = test::get_tmp_test_file(target_test, color)?;
+                let languages = loader.languages_at_path(&current_dir)?;
+                let language = languages
+                    .iter()
+                    .find(|(_, n)| language_names.contains(&Box::from(n.as_str())))
+                    .map(|(l, _)| l.clone());
+                let paths =
+                    collect_paths(None, Some(vec![test_path.to_str().unwrap().to_owned()]))?;
+                (paths, language)
+            } else {
+                (
+                    collect_paths(parse_options.paths_file.as_deref(), parse_options.paths)?,
+                    None,
+                )
+            };
 
             let max_path_length = paths.iter().map(|p| p.chars().count()).max().unwrap_or(0);
             let mut has_error = false;
@@ -579,11 +624,14 @@ fn run() -> Result<()> {
             let should_track_stats = parse_options.stat;
             let mut stats = parse::Stats::default();
 
-            for path in paths {
+            for path in &paths {
                 let path = Path::new(&path);
 
-                let language =
-                    loader.select_language(path, &current_dir, parse_options.scope.as_deref())?;
+                let language = if let Some(ref language) = language {
+                    language.clone()
+                } else {
+                    loader.select_language(path, &current_dir, parse_options.scope.as_deref())?
+                };
                 parser
                     .set_language(&language)
                     .context("incompatible language")?;
@@ -633,12 +681,8 @@ fn run() -> Result<()> {
 
         Commands::Test(test_options) => {
             let config = Config::load(test_options.config_path)?;
-            if test_options.debug {
-                // For augmenting debug logging in external scanners
-                env::set_var("TREE_SITTER_DEBUG", "1");
-            }
 
-            loader.use_debug_build(test_options.debug_build);
+            loader.debug_build(test_options.debug_build);
 
             let mut parser = Parser::new();
 
@@ -646,9 +690,9 @@ fn run() -> Result<()> {
             if test_options.wasm {
                 let engine = tree_sitter::wasmtime::Engine::default();
                 parser
-                    .set_wasm_store(tree_sitter::WasmStore::new(engine.clone()).unwrap())
+                    .set_wasm_store(tree_sitter::WasmStore::new(&engine).unwrap())
                     .unwrap();
-                loader.use_wasm(engine);
+                loader.use_wasm(&engine);
             }
 
             let languages = loader.languages_at_path(&current_dir)?;
@@ -673,6 +717,9 @@ fn run() -> Result<()> {
                     update: test_options.update,
                     open_log: test_options.open_log,
                     languages: languages.iter().map(|(l, n)| (n.as_str(), l)).collect(),
+                    color,
+                    test_num: 1,
+                    show_fields: test_options.show_fields,
                 };
 
                 test::run_tests_at_path(&mut parser, &mut opts)?;
@@ -691,6 +738,7 @@ fn run() -> Result<()> {
                     &config.get()?,
                     &mut highlighter,
                     &test_highlight_dir,
+                    color,
                 )?;
                 parser = highlighter.parser;
             }
@@ -699,10 +747,43 @@ fn run() -> Result<()> {
             if test_tag_dir.is_dir() {
                 let mut tags_context = TagsContext::new();
                 tags_context.parser = parser;
-                test_tags::test_tags(&loader, &config.get()?, &mut tags_context, &test_tag_dir)?;
+                test_tags::test_tags(
+                    &loader,
+                    &config.get()?,
+                    &mut tags_context,
+                    &test_tag_dir,
+                    color,
+                )?;
             }
         }
 
+        Commands::Fuzz(fuzz_options) => {
+            loader.sanitize_build(true);
+
+            let languages = loader.languages_at_path(&current_dir)?;
+            let (language, language_name) = &languages
+                .first()
+                .ok_or_else(|| anyhow!("No language found"))?;
+
+            let mut fuzz_options = FuzzOptions {
+                skipped: fuzz_options.skip,
+                subdir: fuzz_options.subdir,
+                edits: fuzz_options.edits.unwrap_or(*EDIT_COUNT),
+                iterations: fuzz_options.iterations.unwrap_or(*ITERATION_COUNT),
+                filter: fuzz_options.filter,
+                log_graphs: fuzz_options.log_graphs || *LOG_GRAPH_ENABLED,
+                log: fuzz_options.log || *LOG_ENABLED,
+            };
+
+            fuzz_language_corpus(
+                language,
+                language_name,
+                *START_SEED,
+                &current_dir,
+                &mut fuzz_options,
+            );
+        }
+
         Commands::Query(query_options) => {
             let config = Config::load(query_options.config_path)?;
             let paths = collect_paths(query_options.paths_file.as_deref(), query_options.paths)?;
