@@ -1,5 +1,5 @@
 use std::{
-    collections::BTreeMap,
+    collections::{BTreeMap, HashSet},
     ffi::OsStr,
     fs,
     io::{self, Write},
@@ -7,15 +7,15 @@ use std::{
     str,
 };
 
-use ansi_term::Colour;
+use anstyle::{AnsiColor, Color, Style};
 use anyhow::{anyhow, Context, Result};
-use difference::{Changeset, Difference};
 use indoc::indoc;
 use lazy_static::lazy_static;
 use regex::{
     bytes::{Regex as ByteRegex, RegexBuilder as ByteRegexBuilder},
     Regex,
 };
+use similar::{ChangeTag, TextDiff};
 use tree_sitter::{format_sexp, Language, LogType, Parser, Query};
 use walkdir::WalkDir;
 
@@ -27,7 +27,7 @@ lazy_static! {
            (?P<equals>(?:=+){3,})
            (?P<suffix1>[^=\r\n][^\r\n]*)?
            \r?\n
-           (?P<test_name_and_markers>(?:[^=\r\n][^\r\n]*\r?\n)+)
+           (?P<test_name_and_markers>(?:[^=][^\r\n]*\r?\n)+)
            ===+
            (?P<suffix2>[^=\r\n][^\r\n]*)?\r?\n"
     )
@@ -104,6 +104,9 @@ pub struct TestOptions<'a> {
     pub update: bool,
     pub open_log: bool,
     pub languages: BTreeMap<&'a str, &'a Language>,
+    pub color: bool,
+    pub test_num: usize,
+    pub show_fields: bool,
 }
 
 pub fn run_tests_at_path(parser: &mut Parser, opts: &mut TestOptions) -> Result<()> {
@@ -164,12 +167,14 @@ pub fn run_tests_at_path(parser: &mut Parser, opts: &mut TestOptions) -> Result<
                 }
             }
 
-            print_diff_key();
+            if opts.color {
+                print_diff_key();
+            }
             for (i, (name, actual, expected)) in failures.iter().enumerate() {
                 println!("\n  {}. {name}:", i + 1);
                 let actual = format_sexp(actual, 2);
                 let expected = format_sexp(expected, 2);
-                print_diff(&actual, &expected);
+                print_diff(&actual, &expected, opts.color);
             }
 
             if has_parse_errors {
@@ -183,6 +188,65 @@ pub fn run_tests_at_path(parser: &mut Parser, opts: &mut TestOptions) -> Result<
     }
 }
 
+#[allow(clippy::type_complexity)]
+pub fn get_test_info<'test>(
+    test_entry: &'test TestEntry,
+    target_test: u32,
+    test_num: &mut u32,
+) -> Option<(&'test str, &'test [u8], Vec<Box<str>>)> {
+    match test_entry {
+        TestEntry::Example {
+            name,
+            input,
+            attributes,
+            ..
+        } => {
+            if *test_num == target_test {
+                return Some((name, input, attributes.languages.clone()));
+            } else {
+                *test_num += 1;
+            }
+        }
+        TestEntry::Group { children, .. } => {
+            for child in children {
+                if let Some((name, input, languages)) = get_test_info(child, target_test, test_num)
+                {
+                    return Some((name, input, languages));
+                }
+            }
+        }
+    }
+
+    None
+}
+
+/// Writes the input of `target_test` to a temporary file and returns the path
+pub fn get_tmp_test_file(target_test: u32, color: bool) -> Result<(PathBuf, Vec<Box<str>>)> {
+    let current_dir = std::env::current_dir().unwrap();
+    let test_dir = current_dir.join("test").join("corpus");
+
+    // Get the input of the target test
+    let test_entry = parse_tests(&test_dir)?;
+    let mut test_num = 0;
+    let Some((test_name, test_contents, languages)) =
+        get_test_info(&test_entry, target_test - 1, &mut test_num)
+    else {
+        return Err(anyhow!("Failed to fetch contents of test #{target_test}"));
+    };
+
+    // Write the test contents to a temporary file
+    let test_path = std::env::temp_dir().join(".tree-sitter-test");
+    let mut test_file = std::fs::File::create(&test_path)?;
+    test_file.write_all(test_contents)?;
+
+    println!(
+        "{target_test}. {}\n",
+        paint(color.then_some(AnsiColor::Green), test_name)
+    );
+
+    Ok((test_path, languages))
+}
+
 pub fn check_queries_at_path(language: &Language, path: &Path) -> Result<()> {
     if path.exists() {
         for entry in WalkDir::new(path)
@@ -207,29 +271,55 @@ pub fn check_queries_at_path(language: &Language, path: &Path) -> Result<()> {
 pub fn print_diff_key() {
     println!(
         "\ncorrect / {} / {}",
-        Colour::Green.paint("expected"),
-        Colour::Red.paint("unexpected")
+        paint(Some(AnsiColor::Green), "expected"),
+        paint(Some(AnsiColor::Red), "unexpected")
     );
 }
 
-pub fn print_diff(actual: &str, expected: &str) {
-    let changeset = Changeset::new(actual, expected, "\n");
-    for diff in &changeset.diffs {
-        match diff {
-            Difference::Same(part) => {
-                print!("{part}{}", changeset.split);
+pub fn print_diff(actual: &str, expected: &str, use_color: bool) {
+    let diff = TextDiff::from_lines(actual, expected);
+    for diff in diff.iter_all_changes() {
+        match diff.tag() {
+            ChangeTag::Equal => {
+                if use_color {
+                    print!("{diff}");
+                } else {
+                    print!(" {diff}");
+                }
             }
-            Difference::Add(part) => {
-                print!("{}{}", Colour::Green.paint(part), changeset.split);
+            ChangeTag::Insert => {
+                if use_color {
+                    print!("{}", paint(Some(AnsiColor::Green), diff.as_str().unwrap()));
+                } else {
+                    print!("+{diff}");
+                }
+                if diff.missing_newline() {
+                    println!();
+                }
             }
-            Difference::Rem(part) => {
-                print!("{}{}", Colour::Red.paint(part), changeset.split);
+            ChangeTag::Delete => {
+                if use_color {
+                    print!("{}", paint(Some(AnsiColor::Red), diff.as_str().unwrap()));
+                } else {
+                    print!("-{diff}");
+                }
+                if diff.missing_newline() {
+                    println!();
+                }
             }
         }
     }
+
     println!();
 }
 
+pub fn paint(color: Option<AnsiColor>, text: &str) -> String {
+    let style = Style::new().fg_color(color.map(Color::Ansi));
+    format!("{style}{text}{style:#}")
+}
+
+/// This will return false if we want to "fail fast". It will bail and not parse any more tests.
+#[allow(clippy::too_many_arguments)]
 fn run_tests(
     parser: &mut Parser,
     test_entry: TestEntry,
@@ -252,12 +342,20 @@ fn run_tests(
             print!("{}", "  ".repeat(indent_level as usize));
 
             if attributes.skip {
-                println!(" {}", Colour::Yellow.paint(&name));
+                println!(
+                    "{:>3}.  {}",
+                    opts.test_num,
+                    paint(opts.color.then_some(AnsiColor::Yellow), &name),
+                );
                 return Ok(true);
             }
 
             if !attributes.platform {
-                println!(" {}", Colour::Purple.paint(&name));
+                println!(
+                    "{:>3}.  {}",
+                    opts.test_num,
+                    paint(opts.color.then_some(AnsiColor::Magenta), &name),
+                );
                 return Ok(true);
             }
 
@@ -273,9 +371,17 @@ fn run_tests(
 
                 if attributes.error {
                     if tree.root_node().has_error() {
-                        println!(" {}", Colour::Green.paint(&name));
+                        println!(
+                            "{:>3}.  {}",
+                            opts.test_num,
+                            paint(opts.color.then_some(AnsiColor::Green), &name)
+                        );
                     } else {
-                        println!(" {}", Colour::Red.paint(&name));
+                        println!(
+                            "{:>3}.  {}",
+                            opts.test_num,
+                            paint(opts.color.then_some(AnsiColor::Red), &name)
+                        );
                     }
 
                     if attributes.fail_fast {
@@ -283,12 +389,16 @@ fn run_tests(
                     }
                 } else {
                     let mut actual = tree.root_node().to_sexp();
-                    if !has_fields {
+                    if !(opts.show_fields || has_fields) {
                         actual = strip_sexp_fields(&actual);
                     }
 
                     if actual == output {
-                        println!("✓ {}", Colour::Green.paint(&name));
+                        println!(
+                            "{:>3}. ✓ {}",
+                            opts.test_num,
+                            paint(opts.color.then_some(AnsiColor::Green), &name)
+                        );
                         if opts.update {
                             let input = String::from_utf8(input.clone()).unwrap();
                             let output = format_sexp(&output, 0);
@@ -330,53 +440,77 @@ fn run_tests(
                                     header_delim_len,
                                     divider_delim_len,
                                 ));
-                                println!("✓ {}", Colour::Blue.paint(&name));
+                                println!(
+                                    "{:>3}. ✓ {}",
+                                    opts.test_num,
+                                    paint(opts.color.then_some(AnsiColor::Blue), &name),
+                                );
                             }
                         } else {
-                            println!("✗ {}", Colour::Red.paint(&name));
+                            println!(
+                                "{:>3}. ✗ {}",
+                                opts.test_num,
+                                paint(opts.color.then_some(AnsiColor::Red), &name),
+                            );
                         }
                         failures.push((name.clone(), actual, output.clone()));
 
                         if attributes.fail_fast {
-                            // return value of false means to fail fast
                             return Ok(false);
                         }
-
-                        if i == attributes.languages.len() - 1 {
-                            // reset back to first language
-                            parser.set_language(opts.languages.values().next().unwrap())?;
-                        }
                     }
                 }
+
+                if i == attributes.languages.len() - 1 {
+                    // reset to the first language
+                    parser.set_language(opts.languages.values().next().unwrap())?;
+                }
             }
+            opts.test_num += 1;
         }
         TestEntry::Group {
             name,
             mut children,
             file_path,
         } => {
-            children.retain(|child| {
-                if let TestEntry::Example { name, .. } = child {
+            // track which tests are being skipped to maintain consistent numbering while using
+            // filters
+            let mut skipped_tests = HashSet::new();
+            let mut advance_counter = opts.test_num;
+            children.retain(|child| match child {
+                TestEntry::Example { name, .. } => {
                     if let Some(filter) = opts.filter {
                         if !name.contains(filter) {
+                            skipped_tests.insert(advance_counter);
+                            advance_counter += 1;
                             return false;
                         }
                     }
                     if let Some(include) = &opts.include {
                         if !include.is_match(name) {
+                            skipped_tests.insert(advance_counter);
+                            advance_counter += 1;
                             return false;
                         }
                     }
                     if let Some(exclude) = &opts.exclude {
                         if exclude.is_match(name) {
+                            skipped_tests.insert(advance_counter);
+                            advance_counter += 1;
                             return false;
                         }
                     }
+                    advance_counter += 1;
+                    true
+                }
+                TestEntry::Group { .. } => {
+                    advance_counter += count_subtests(child);
+                    true
                 }
-                true
             });
 
             if children.is_empty() {
+                opts.test_num = advance_counter;
                 return Ok(true);
             }
 
@@ -389,6 +523,11 @@ fn run_tests(
 
             indent_level += 1;
             for child in children {
+                if let TestEntry::Example { .. } = child {
+                    while skipped_tests.remove(&opts.test_num) {
+                        opts.test_num += 1;
+                    }
+                }
                 if !run_tests(
                     parser,
                     child,
@@ -403,6 +542,8 @@ fn run_tests(
                 }
             }
 
+            opts.test_num += skipped_tests.len();
+
             if let Some(file_path) = file_path {
                 if opts.update && failures.len() - failure_count > 0 {
                     write_tests(&file_path, corrected_entries)?;
@@ -414,6 +555,15 @@ fn run_tests(
     Ok(true)
 }
 
+fn count_subtests(test_entry: &TestEntry) -> usize {
+    match test_entry {
+        TestEntry::Example { .. } => 1,
+        TestEntry::Group { children, .. } => children
+            .iter()
+            .fold(0, |count, child| count + count_subtests(child)),
+    }
+}
+
 fn write_tests(
     file_path: &Path,
     corrected_entries: &[(String, String, String, usize, usize)],
